package internal

import (
	"crypto/sha256"
	"encoding/hex"
	"fmt"
	"log"
	"net/http"

	"github.com/go-resty/resty/v2"
	"github.com/tidwall/gjson"
)

type RC struct {
	*resty.Client
}

func NewRC(host, username, password string) RC {
	if host == "" {
		log.Fatal("host  is empty")
	}
	if username == "" || password == "" {
		log.Fatal("username or password is empty")
	}
	rc := RC{
		Client: resty.New().
			SetBaseURL(host).
			SetHeader("Content-Type", "application/json"),
	}
	log.Printf("connecting to %s\n", host)
	h := sha256.New()
	h.Write([]byte(password))
	rc.Client = rc.Client.
		SetHeader("X-2fa-method", "password").
		SetHeader("X-2fa-code", hex.EncodeToString(h.Sum(nil)))
	rc.Login(username, password)
	log.Println("authenticated")

	return rc
}

func (rc *RC) Login(username, password string) {
	body := map[string]string{
		"user":     username,
		"password": password,
	}
	resp, err := rc.R().SetBody(body).Post("/api/v1/login")
	ExitOnError(err)
	if resp.StatusCode() == http.StatusUnauthorized {
		log.Fatal("invalid credentials")
	}
	ExitOnResponseError(resp)
	data := gjson.GetMany(string(resp.Body()), "data.userId", "data.authToken")
	rc.Client = rc.Client.
		SetHeader("X-User-Id", data[0].String()).
		SetHeader("X-Auth-Token", data[1].String())
}

func (rc RC) SetAppDevMode(value bool) {
	body := fmt.Sprintf(`{"message":"{\"msg\":\"method\",\"id\":\"30\",\"method\":\"saveSettings\",\"params\":[[{\"_id\":\"Apps_Framework_Development_Mode\",\"value\":%t}]]}"}`, value)
	resp, err := rc.R().SetBody(body).Post("/api/v1/method.call/saveSettings")
	ExitOnError(err)
	ExitOnResponseError(resp)
}

func (rc RC) IsInstalled(app App) bool {
	resp, err := rc.R().Get(fmt.Sprintf("/api/apps/%s", app.ID))
	ExitOnError(err)
	return resp.IsSuccess()
}

func (rc RC) Install(app App) {
	resp, err := rc.R().SetFile("app", app.Filename).Post("/api/apps")
	ExitOnError(err)
	ExitOnResponseError(resp)
	log.Printf("%s (%s) installed", app.Name, app.ID)
}

func (rc RC) Update(app App) {
	resp, err := rc.R().SetFile("app", app.Filename).Post("/api/apps/update")
	ExitOnError(err)
	ExitOnResponseError(resp)
	log.Printf("%s (%s) updated", app.Name, app.ID)
}

func (rc RC) Uninstall(id string) {
	resp, err := rc.R().Delete(fmt.Sprintf("/api/apps/%s", id))
	ExitOnError(err)
	ExitOnResponseError(resp)
	log.Printf("%s  uninstalled", id)
}

func (rc RC) List() []string {
	resp, err := rc.R().Get("/api/apps")
	ExitOnError(err)
	ExitOnResponseError(resp)
	list := []string{}
	gjson.Get(string(resp.Body()), "apps.#.id").ForEach(func(key, value gjson.Result) bool {
		list = append(list, value.String())
		return true
	})
	return list
}
